
import numpy as np
from wuml.type_check import *

from sklearn.preprocessing import OneHotEncoder
from sklearn.preprocessing import LabelEncoder

def double_center(Ψ):
	Ψ = ensure_numpy(Ψ)
	HΨ = Ψ - np.mean(Ψ, axis=0)								# equivalent to Γ = Ⲏ.dot(Kᵧ).dot(Ⲏ)
	HΨH = (HΨ.T - np.mean(HΨ.T, axis=0)).T
	return HΨH

def get_np_column(data, column_id):
	X = ensure_numpy(data)
	return np.atleast_2d(X[:,column_id]).T

def sort_matrix_rows_by_a_column(X, column_names):
	df = ensure_DataFrame(X)
	sorted_df = df.sort_values(by=column_names)

	return sorted_df

def one_hot_encoding(Y, output_data_type='same'): 
	'''
		output_data_type : 'same' implies to use the same data type as input
							else use the data type you want, i.e., 'ndarray' , 'Tensor', 'DataFrame'
	'''
	Y1 = wuml.ensure_numpy(Y)

	Y1 = np.reshape(Y1,(len(Y1),1))
	Yₒ = OneHotEncoder(categories='auto', sparse=False).fit_transform(Y1)

	if output_data_type == 'same':
		Yₒ = wuml.ensure_data_type(Yₒ, type_name=wtype(Y))
	else:
		Yₒ = wuml.ensure_data_type(Yₒ, type_name=output_data_type)

	return Yₒ

def one_hot_to_label(Yₒ):
	Y = np.argmax(Yₒ, axis=1)
	return Y

def compute_Degree_matrix(M, output_type=None):
	if output_type is None: output_type = wtype(M)

	Mnp = ensure_numpy(M)
	D = np.diag(np.sum(Mnp, axis=0))

	return ensure_data_type(D, type_name=output_type)





from scipy.stats import multivariate_normal
import numpy as np
import wuml

class multivariate_gaussian: #Assuming that data is 1 dimension
	def __init__(self, dat=None, mean=None, cov=None):
		self.X = X = wuml.ensure_numpy(dat)	

		if X is not None:
			μ = np.mean(X, axis=0)
			Σ = np.corrcoef(X.T)

			self.pₓ = multivariate_normal(mean=μ, cov=Σ)

		elif mean is not None and cov is not None:
			self.pₓ = multivariate_normal(mean=mean, cov=cov)

	def generate_samples(self, num_of_samples, return_data_type='wData'):
		dat = self.pₓ.rvs(size=num_of_samples, random_state=None)
		return ensure_data_type(dat, type_name=return_data_type)


	def __call__(self, data, return_log_likelihood=False, return_data_type='wData'):
		inp = wuml.ensure_numpy(data)	
		prob = self.pₓ.pdf(inp)

		if return_log_likelihood: output = np.log(prob)
		else: output = prob

		return ensure_data_type(output, type_name=return_data_type)


class exponential: #Assuming that data is 1 dimension
	def __init__(self, X=None, mean=None):
		if X is not None:
			X = np.squeeze(wuml.ensure_numpy(X))
			self.λ = 1/np.mean(X, axis=0)
		elif mean is not None:
			self.λ = 1/mean

	def __call__(self, x):
		try: x[x<0]=np.NAN
		except: pass

		λ = self.λ
		S = λ*np.exp(-λ*x)

		try: S[np.isnan(S)] = 0
		except: pass

		return S


	def generate_samples(size=1):
		# λ exp(-λx) , μ=1/λ
		return np.random.exponential(scale=self.λ, size=size)



	def cdf(self,x):
		λ = self.λ
		return (1 - np.exp(-λ*x))

		#[Area, error] = wuml.integrate(self.__call__, 0, x)
		#return Area

